#!/bin/bash

# include set_config_param()
. scripts/bareos-config-lib.sh

replace_password()
{
    local var=$1
    local file=$2
    PASSWORD='@!@\
with open("/etc/bareos/.rndpwd","r") as f:\
  for l in f.readlines():\
    if "XXX_REPLACE_TEMP_MARKER_XXX" in l:\
      print (l.split("=")[1].strip()),\
@!@'
    sed -i -e "s|\"$var\"|$PASSWORD|" -e "s|XXX_REPLACE_TEMP_MARKER_XXX|$var|" $file
}

create_bareos_dir_template()
{
    local file=$DEST_DIR/etc/bareos/bareos-dir.conf
    mkdir -p `dirname $file`
    printf '%s\n' '@%@UCRWARNING=# @%@' > $file
    cat src/defaultconfigs/diskonly/bareos-dir.conf >> $file
    sed -i 's/XXX_REPLACE_WITH_LOCAL_HOSTNAME_XXX/@%@hostname@%@.@%@domainname@%@/' $file
    replace_password XXX_REPLACE_WITH_DIRECTOR_PASSWORD_XXX $file
    replace_password XXX_REPLACE_WITH_DIRECTOR_MONITOR_PASSWORD_XXX $file
    replace_password XXX_REPLACE_WITH_STORAGE_PASSWORD_XXX $file
    replace_password XXX_REPLACE_WITH_CLIENT_PASSWORD_XXX $file

    set_config_param $file Catalog MyCatalog    dbdriver "postgresql"
    set_config_param $file JobDefs DefaultJob   FileSet "\"Linux All\""
    set_config_param $file Pool    Full         MaximumVolumeBytes '@%@bareos/max_full_volume_bytes@%@ G'
    set_config_param $file Pool    Full         MaximumVolumes     '@%@bareos/max_full_volumes@%@'
    set_config_param $file Pool    Differential MaximumVolumeBytes '@%@bareos/max_diff_volume_bytes@%@ G'
    set_config_param $file Pool    Differential MaximumVolumes     '@%@bareos/max_diff_volumes@%@'
    set_config_param $file Pool    Incremental  MaximumVolumeBytes '@%@bareos/max_incr_volume_bytes@%@ G'
    set_config_param $file Pool    Incremental  MaximumVolumes     '@%@bareos/max_incr_volumes@%@'

    printf '%s\n' '
FileSet {
  Name = "UCS PDC"
  Include {
    Options {
      Signature = MD5 # calculate md5 checksum per file
      One FS = No     # change into other filessytems
      FS Type = ext2  # filesystems of given types will be backed up
      FS Type = ext3  # others will be ignored
      FS Type = ext4
      FS Type = xfs
    }
    File = /
  }
  # Things that usually have to be excluded
  # You have to exclude /var/lib/bareos/storage
  # on your bareos server
  Exclude {
    File = /var/lib/bareos
    File = /var/lib/bareos/storage
    File = /proc
    File = /tmp
    File = /.journal
    File = /.fsck
  }
}

Job {
    Name     = "Backup-@%@hostname@%@.@%@domainname@%@"
    JobDefs  = "DefaultJob"
    FileSet  = "UCS PDC"
    Enabled  = "@%@bareos/backup_myself@%@"
}

Job {
    Name     = "Backup-SelfTest"
    Client   = @%@hostname@%@.@%@domainname@%@-fd
    Type     = Backup
    Level    = Incremental
    FileSet  = "SelfTest"
    Storage  = File
    Messages = Standard
    Pool     = Incremental
    Full Backup Pool         = Full
    Differential Backup Pool = Differential
    Incremental Backup Pool  = Incremental
}

# include clients configured in webfrontend
@/etc/bareos/autogenerated/clients.include

#
# bareos-webui
#
@/etc/bareos/bareos-dir.d/webui-profiles.conf

# Restricted console used by bareos-webui
Console {
  Name = @%@bareos/webui/console/user1/username@%@
  Password = "@%@bareos/webui/console/user1/password@%@"
  Profile = webui
}

' >> $file

    cat $file
}

create_bareos_sd_template()
{
    local file=$DEST_DIR/etc/bareos/bareos-sd.conf
    mkdir -p `dirname $file`
    printf '%s\n' '@%@UCRWARNING=# @%@' > $file
    cat src/defaultconfigs/diskonly/bareos-sd.conf >> $file
    sed -i 's/XXX_REPLACE_WITH_LOCAL_HOSTNAME_XXX/@%@hostname@%@.@%@domainname@%@/' $file
    replace_password XXX_REPLACE_WITH_STORAGE_PASSWORD_XXX $file
    replace_password XXX_REPLACE_WITH_STORAGE_MONITOR_PASSWORD_XXX $file
    #sed -i 's/ *Archive Device.*$/Archive Device = @%@bareos/filestorage@%@/' $file
    set_config_param $file Device FileStorage ArchiveDevice '@%@bareos/filestorage@%@'
    cat $file
}

create_bareos_fd_template()
{
    local file=$DEST_DIR/etc/bareos/bareos-fd.conf
    mkdir -p `dirname $file`
    printf '%s\n' '@%@UCRWARNING=# @%@' > $file
    cat src/filed/bareos-fd.conf >> $file
    sed -i 's/XXX_REPLACE_WITH_LOCAL_HOSTNAME_XXX/@%@hostname@%@.@%@domainname@%@/' $file
    replace_password XXX_REPLACE_WITH_CLIENT_PASSWORD_XXX $file
    replace_password XXX_REPLACE_WITH_CLIENT_MONITOR_PASSWORD_XXX $file
    cat $file
}

create_bconsole_template()
{
    local file=$DEST_DIR/etc/bareos/bconsole.conf
    mkdir -p `dirname $file`
    printf '%s\n' '@%@UCRWARNING=# @%@' > $file
    cat src/console/bconsole.conf >> $file
    sed -i 's/XXX_REPLACE_WITH_LOCAL_HOSTNAME_XXX/@%@hostname@%@.@%@domainname@%@/' $file
    replace_password XXX_REPLACE_WITH_DIRECTOR_PASSWORD_XXX $file
    cat $file
}

DEST_DIR=$1
create_bareos_fd_template
create_bareos_sd_template
create_bareos_dir_template
create_bconsole_template
